package com.example.bookingapp_austinblanke;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import java.util.ArrayList;
import java.util.Random;


public class DatabaseConnector extends SQLiteOpenHelper {

    private static final String DB_NAME = "Registration.db";
    private static final int DB_VERSION = 1;
    private static final String TABLE_USERS = "Users";
    private static final String COLUMN_UID = "uid";
    private static final String COLUMN_USERNAME = "username";
    private static final String COLUMN_PASSWORD = "password";

    private static final String TABLE_PERFORMANCES = "performances";
    private static final String COLUMN_PERFORMANCEID = "performanceID";
    private static final String COLUMN_NAME = "name";
    private static final String COLUMN_DATE = "date";
    private static final String COLUMN_TIME = "time";
    private static final String COLUMN_VENUE = "venue";
    private static final String COLUMN_FLASH = "flash";
    private static final String COLUMN_WHEELCHAIR = "wheelchair";
    private static final String COLUMN_STEPS = "steps";
    private static final String COLUMN_TICKET1 = "performanceTicket1";
    private static final String COLUMN_TICKET1TYPE = "performanceTicketType1";
    private static final String COLUMN_TICKET1CNT = "performanceTicketCount1";
    private static final String COLUMN_TICKET2 = "performanceTicket2";
    private static final String COLUMN_TICKET2TYPE = "performanceTicketType2";
    private static final String COLUMN_TICKET2CNT = "performanceTicketCount2";
    private static final String COLUMN_TICKET3 = "performanceTicket3";
    private static final String COLUMN_TICKET3TYPE = "performanceTicketType3";
    private static final String COLUMN_TICKET3CNT = "performanceTicketCount3";


    private static final String TABLE_BOOKINGS = "bookings";
    private static final String COLUMN_BID = "BID";
    private static final String COLUMN_PID = "PID";
    private static final String COLUMN_TIC1CNT = "ticket1count";
    private static final String COLUMN_TIC2CNT = "ticket2count";
    private static final String COLUMN_TIC3CNT = "ticket3count";
    private static final String COLUMN_BT = "bt";


    public DatabaseConnector(Context context, SQLiteDatabase.CursorFactory factory) {
        super(context, DB_NAME, factory, DB_VERSION);
    }

    @Override
    public void onCreate(SQLiteDatabase db) {

        String CREATE_table1 = "CREATE TABLE " + TABLE_USERS + "(" +
                COLUMN_UID + " INTEGER PRIMARY KEY, " +
                COLUMN_USERNAME + " TEXT, " +
                COLUMN_PASSWORD + " TEXT)";
                ;
        db.execSQL(CREATE_table1);

        String CREATE_table2 = "CREATE TABLE " + TABLE_PERFORMANCES + "(" +
                COLUMN_PERFORMANCEID + " TEXT PRIMARY KEY, " +
                COLUMN_NAME + " TEXT, " +
                COLUMN_DATE + " TEXT, " +
                COLUMN_TIME + " TEXT, " +
                COLUMN_VENUE + " TEXT, " +
                COLUMN_FLASH + " TEXT, " +
                COLUMN_WHEELCHAIR + " TEXT, " +
                COLUMN_STEPS + " TEXT, " +
                COLUMN_TICKET1 + " INT, " +
                COLUMN_TICKET1TYPE + " TEXT, " +
                COLUMN_TICKET1CNT + " INT, " +
                COLUMN_TICKET2 + " INT, " +
                COLUMN_TICKET2TYPE + " TEXT, " +
                COLUMN_TICKET2CNT + " INT, " +
                COLUMN_TICKET3 + " INT, " +
                COLUMN_TICKET3TYPE + " TEXT, " +
                COLUMN_TICKET3CNT + " INT)";
        db.execSQL(CREATE_table2);

        String CREATE_table3 = "CREATE TABLE " + TABLE_BOOKINGS + "(" +
                COLUMN_BID + " INTEGER PRIMARY KEY, " +
                COLUMN_PID + " TEXT, " +
                COLUMN_TIC1CNT + " INT, " +
                COLUMN_TIC2CNT + " INT, " +
                COLUMN_TIC3CNT + " INT, " +
                COLUMN_BT + " TEXT)";
        db.execSQL(CREATE_table3);

        ContentValues play1 = new ContentValues();
        play1.put(COLUMN_PERFORMANCEID, "1");
        play1.put(COLUMN_NAME, "The Merchant of Venice by William Shakespeare");
        play1.put(COLUMN_DATE, "Sunday April 23 2023");
        play1.put(COLUMN_TIME, "20:00");
        play1.put(COLUMN_VENUE, "Merchant Adventurer's Hall, York");
        play1.put(COLUMN_FLASH, "false");
        play1.put(COLUMN_WHEELCHAIR, "true");
        play1.put(COLUMN_STEPS, "false");
        play1.put(COLUMN_TICKET1CNT, 17);
        play1.put(COLUMN_TICKET1, 8);
        play1.put(COLUMN_TICKET1TYPE, "Seated");
        db.insert(TABLE_PERFORMANCES, null, play1);

        ContentValues play2 = new ContentValues();
        play2.put(COLUMN_PERFORMANCEID, "2");
        play2.put(COLUMN_NAME, "Hamlet by William Shakespeare");
        play2.put(COLUMN_DATE, "Friday 19 May 2023");
        play2.put(COLUMN_TIME, "18:00");
        play2.put(COLUMN_VENUE, "Clifford's Tower, York");
        play2.put(COLUMN_FLASH, "true");
        play2.put(COLUMN_WHEELCHAIR, "false");
        play2.put(COLUMN_STEPS, "true");
        play2.put(COLUMN_TICKET1CNT, 8);
        play2.put(COLUMN_TICKET1, 8);
        play2.put(COLUMN_TICKET1TYPE, "Seated");
        play2.put(COLUMN_TICKET2CNT, 12);
        play2.put(COLUMN_TICKET2, 7);
        play2.put(COLUMN_TICKET2TYPE, "Standing");
        db.insert(TABLE_PERFORMANCES, null, play2);

        ContentValues play3 = new ContentValues();
        play3.put(COLUMN_PERFORMANCEID, "3");
        play3.put(COLUMN_NAME, "A Midsummer Night's Dream by William Shakespeare");
        play3.put(COLUMN_DATE, "Saturday 3 June 2023");
        play3.put(COLUMN_TIME, "19:00");
        play3.put(COLUMN_VENUE, "Dean's Park, York");
        play3.put(COLUMN_FLASH, "true");
        play3.put(COLUMN_WHEELCHAIR, "true");
        play3.put(COLUMN_STEPS, "false");
        play3.put(COLUMN_TICKET1TYPE, "Stage");
        play3.put(COLUMN_TICKET1CNT, 11);
        play3.put(COLUMN_TICKET1, 4);
        play3.put(COLUMN_TICKET1TYPE, "Grass");
        play3.put(COLUMN_TICKET2CNT, 29);
        play3.put(COLUMN_TICKET2, 4);
        db.insert(TABLE_PERFORMANCES, null, play3);

        ContentValues play4 = new ContentValues();
        play4.put(COLUMN_PERFORMANCEID, "4");
        play4.put(COLUMN_NAME, "Oedipus the King by Sophocles");
        play4.put(COLUMN_DATE, "Friday 28 July");
        play4.put(COLUMN_TIME, "20:00");
        play4.put(COLUMN_VENUE, "St Mary's Abbey, Museum Gardens, York");
        play4.put(COLUMN_FLASH, "false");
        play4.put(COLUMN_WHEELCHAIR, "true");
        play4.put(COLUMN_STEPS, "false");
        play4.put(COLUMN_TICKET1TYPE, "Seated");
        play4.put(COLUMN_TICKET1CNT, 5);
        play4.put(COLUMN_TICKET1, 9);
        play4.put(COLUMN_TICKET1TYPE, "Standing");
        play4.put(COLUMN_TICKET2CNT, 12);
        play4.put(COLUMN_TICKET2, 7);
        db.insert(TABLE_PERFORMANCES, null, play4);

        ContentValues play5 = new ContentValues();
        play5.put(COLUMN_PERFORMANCEID, "5");
        play5.put(COLUMN_NAME, "The Tempest by William Shakespeare");
        play5.put(COLUMN_DATE, "Saturday 19 August");
        play5.put(COLUMN_TIME, "14:00");
        play5.put(COLUMN_VENUE, "Millennium Bridge, York");
        play5.put(COLUMN_FLASH, "false");
        play5.put(COLUMN_WHEELCHAIR, "true");
        play5.put(COLUMN_STEPS, "false");
        play5.put(COLUMN_TICKET1TYPE, "Boat A");
        play5.put(COLUMN_TICKET1CNT, 6);
        play5.put(COLUMN_TICKET1, 9);
        play5.put(COLUMN_TICKET1TYPE, "Boat B");
        play5.put(COLUMN_TICKET2CNT, 10);
        play5.put(COLUMN_TICKET2, 7);
        db.insert(TABLE_PERFORMANCES, null, play5);

        ContentValues play6 = new ContentValues();
        play6.put(COLUMN_PERFORMANCEID, "6");
        play6.put(COLUMN_NAME, "Antigone by Sophocles");
        play6.put(COLUMN_DATE, "Thursday 20 September");
        play6.put(COLUMN_TIME, "21:00");
        play6.put(COLUMN_VENUE, "Crypt, York Minster, York");
        play6.put(COLUMN_FLASH, "false");
        play6.put(COLUMN_WHEELCHAIR, "false");
        play6.put(COLUMN_STEPS, "true");
        play6.put(COLUMN_TICKET1TYPE, "Inner Circle");
        play6.put(COLUMN_TICKET1CNT, 5);
        play6.put(COLUMN_TICKET1, 16);
        play6.put(COLUMN_TICKET1TYPE, "Outer Circle");
        play6.put(COLUMN_TICKET2CNT, 8);
        play6.put(COLUMN_TICKET2, 13);
        play6.put(COLUMN_TICKET1TYPE, "Standing");
        play6.put(COLUMN_TICKET3CNT, 10);
        play6.put(COLUMN_TICKET3, 10);
        db.insert(TABLE_PERFORMANCES, null, play6);

    }

    public void addNewUser(String username, String password) {
        ContentValues values = new ContentValues();
        values.put(COLUMN_USERNAME, username);
        values.put(COLUMN_PASSWORD, password);
        SQLiteDatabase db = this.getWritableDatabase();
        db.insert(TABLE_USERS, null, values);
        db.close();
    }

    public void addBooking(String PID, int TIC1CNT, int TIC2CNT, int TIC3CNT) {

        String CurrentPTicCount1 = null;
        String CurrentPTicCount2 = null;
        String CurrentPTicCount3 = null;

        String query = "SELECT * FROM " + TABLE_PERFORMANCES + " WHERE " + COLUMN_PERFORMANCEID + " = \"" + PID + "\"";
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(query, null);
        cursor.moveToFirst();
        String title = "\'" + cursor.getString(1) + "\'";
        CurrentPTicCount1 = cursor.getString(10);
        CurrentPTicCount2 = cursor.getString(13);
        CurrentPTicCount3 = cursor.getString(16);
        cursor.close();

        if(CurrentPTicCount1 == null){CurrentPTicCount1 = "0";}
        if(CurrentPTicCount2 == null){CurrentPTicCount2 = "0";}
        if(CurrentPTicCount3 == null){CurrentPTicCount3 = "0";}

        if(CurrentPTicCount1 != "0"){CurrentPTicCount1 = String.valueOf(Integer.parseInt(CurrentPTicCount1) - TIC1CNT);}
        if(CurrentPTicCount2 != "0"){CurrentPTicCount2 = String.valueOf(Integer.parseInt(CurrentPTicCount2) - TIC2CNT);}
        if(CurrentPTicCount3 != "0"){CurrentPTicCount3 = String.valueOf(Integer.parseInt(CurrentPTicCount3) - TIC3CNT);}

        System.out.println(CurrentPTicCount1);
        System.out.println(CurrentPTicCount2);
        System.out.println(CurrentPTicCount3);

        ContentValues values2 = new ContentValues();
        values2.put(COLUMN_TICKET1CNT, CurrentPTicCount1);
        String whereClause = COLUMN_PERFORMANCEID + " = " + "\"" + PID + "\"";
        db.update(TABLE_PERFORMANCES, values2, whereClause, null);

        Random rand = new Random();
        int BID = rand.nextInt(10000);

        ContentValues values = new ContentValues();
        values.put(COLUMN_BID, BID);
        values.put(COLUMN_PID, PID);
        values.put(COLUMN_TIC1CNT, TIC1CNT);
        values.put(COLUMN_TIC2CNT, TIC2CNT);
        values.put(COLUMN_TIC3CNT, TIC3CNT);
        values.put(COLUMN_BT, title);
        db.insert(TABLE_BOOKINGS, null, values);
        db.close();
    }

    @Override
    public void onUpgrade(SQLiteDatabase sqLiteDatabase, int i, int i1) {

    }

    public boolean usernameRepeat(String username) {
        boolean result = false;
        String query = "SELECT * FROM " + TABLE_USERS + " WHERE " + COLUMN_USERNAME + " = \"" + username + "\"";
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(query, null);

        if (cursor.getCount() == 0) {
            result = false;
        } else result = true;


        cursor.close();
        db.close();

        return result;
    }

    public String getTicketCount(String PID, int t) {
        String result = null;
        String query = "SELECT * FROM " + TABLE_PERFORMANCES + " WHERE " + COLUMN_PERFORMANCEID + " = \"" + PID + "\"";
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(query, null);
        cursor.moveToFirst();
        if(t==1){result=cursor.getString(10);}
        else if(t==2){result=cursor.getString(13);}
        else if(t==3){result=cursor.getString(16);}
        return result;
    }

    public String getTicketPrice(String PID, int t) {
        String result = null;
        String query = "SELECT * FROM " + TABLE_PERFORMANCES + " WHERE " + COLUMN_PERFORMANCEID + " = \"" + PID + "\"";
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(query, null);
        cursor.moveToFirst();
        if(t==1){result=cursor.getString(8);}
        else if(t==2){result=cursor.getString(11);}
        else if(t==3){result=cursor.getString(14);}
        return result;
    }

    public String getTicketType(String PID, int t) {
        String result = null;
        String query = "SELECT * FROM " + TABLE_PERFORMANCES + " WHERE " + COLUMN_PERFORMANCEID + " = \"" + PID + "\"";
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(query, null);
        cursor.moveToFirst();
        if(t==1){result=cursor.getString(9);}
        else if(t==2){result=cursor.getString(12);}
        else if(t==3){result=cursor.getString(15);}
        return result;
    }



    public String getInfo(String PID, int column) {
        String result;
        String query = "SELECT * FROM " + TABLE_PERFORMANCES + " WHERE " + COLUMN_PERFORMANCEID + " = \"" + PID + "\"";
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(query, null);
        cursor.moveToFirst();
        result = cursor.getString(column).toString();
        cursor.close();
        db.close();
        return result;
    }
    public boolean login(String username, String password) {
        String query = "SELECT * FROM " + TABLE_USERS + " WHERE " + COLUMN_USERNAME + " = \"" + username + "\"";
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(query, null);

        boolean result = false;

        if (cursor.moveToFirst()) {
            if (cursor.getString(2).equals(password))
                result = true;
            else
                result = false;
        }

        cursor.close();
        db.close();

        return result;
    }

    public int overBook(String PID, int Cnt){
        int result = 0;
        String query = "SELECT * FROM " + TABLE_PERFORMANCES + " WHERE " + COLUMN_PERFORMANCEID + " = \"" + PID + "\"";
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(query, null);
        cursor.moveToFirst();
        if(Cnt == 1){result = Integer.parseInt(cursor.getString(10));}
        if(Cnt == 2){result = Integer.parseInt(cursor.getString(13));}
        if(Cnt == 3){result = Integer.parseInt(cursor.getString(16));}
        cursor.close();
        db.close();
        return result;
    }

    public void deleteBooking(int BID){
        String query = "SELECT * FROM " + TABLE_BOOKINGS;
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(query, null);
        db.delete(TABLE_BOOKINGS, COLUMN_BID + "=" + BID, null);
        cursor.close();
        db.close();
    }
    public ArrayList bookings(){
        ArrayList<String> bookings = new ArrayList<>();
        String query = "SELECT * FROM " + TABLE_BOOKINGS;
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(query, null);
        cursor.moveToFirst();
        for(int i=0;i< cursor.getCount();i++){
            bookings.add(cursor.getString(5) + "id: " + cursor.getString(0));
            cursor.moveToNext();
        }
        return bookings;
    }
    public ArrayList performanceTitles(boolean filter1, boolean filter2, boolean filter3) {

        ArrayList<String> performances = new ArrayList<>();
        String query = "SELECT * FROM " + TABLE_PERFORMANCES;
        SQLiteDatabase db = this.getWritableDatabase();
        Cursor cursor = db.rawQuery(query, null);
        cursor.moveToFirst();
        for(int i=0;i< cursor.getCount();i++){
            if (filter1) {
                if (cursor.getString(5).equals("false")) {
                    performances.add(cursor.getString(1) + " id: " +cursor.getString(0));
                }
            }
            else if(filter2) {
                if (cursor.getString(6).equals("true")) {
                    performances.add(cursor.getString(1) + " id: " +cursor.getString(0));
                }
            }
            else if(filter3) {
                if (cursor.getString(7).equals("false")) {
                    performances.add(cursor.getString(1) + " id: " +cursor.getString(0));
                }
            }
            else{performances.add(cursor.getString(1) + " id: " +cursor.getString(0));} //No filter present
            cursor.moveToNext();
        }

        //Close DB
        cursor.close();
        db.close();

        return performances;
    }

}